"""


The rest of the DDoS tools have been removed and will be added slowly in the coming versions :) Be patient !!


"""
from bane.ddos.http_spam  import *
from bane.ddos.proxies_hammer  import *
from bane.ddos.proxies_http_spam  import *
from bane.ddos.proxies_xerxes  import *
from bane.ddos.slow_read  import *
from bane.ddos.tcp  import *
from bane.ddos.torshammer  import *
from bane.ddos.udp  import *
from bane.ddos.vse  import *
from bane.ddos.xerxes  import *
